function ItemScore:ParseTag(name, format, tag)
	if format == "AskMrRobot" then
		self:ParseMrRobotFormat(name, tag)
	elseif format == "Pawn" then
		self:ParsePawnFormat(tag)
	end
end

function ItemScore:ExportSet(setName, format)
	if format == "AskMrRobot" then
		return self:ExportMrRobotFormat(setName)
	elseif format == "Pawn" then
		return self:ExportPawnFormat(setName)
	end
	
	return ""
end

local mrRobotToStats = {
	["CriticalStrike"] = "MELEE_CRIT_RATING",
	["Mastery"] = "MASTERY_RATING",
	["Haste"] = "MELEE_HASTE_RATING",
	["SpellPower"] = "SPELL_POWER", 
	["AttackPower"] = "AP",
	["MainHandDps"] = "DPS",
	["BonusArmor"] = "ARMOR_BONUS",
	["Avoidance"] = "AVOIDANCE",
	["Leech"] = "LEECH",
	["MovementSpeed"] = "RUN_SPEED",
}

local statsToMrRobot = {}
	
-- now set the values of each index to readable values from stat logic
for i, v in pairs(mrRobotToStats) do
	statsToMrRobot[v] = i
end

function ItemScore:ParseMrRobotFormat(name, tag)
	local values = {}
	for k, v in string.gmatch(tag, "(%w+) +(%d\.?%d*)") do
		values[k] = v
	end
	
	self:CreateNewSet(name)
	for k, v in pairs(values) do
		-- try some hard-coded values first
		local statName = mrRobotToStats[k]
		
		if not statName then
			statName = ItemScore.reverseStats[k]
		end
		
		if not statName then
			DEFAULT_CHAT_FRAME:AddMessage("Unsupported stat '" .. k .. "' ignored.")
		elseif self:AddStat(name, statName) then
			self:SetScore(name, statName, v)
		end
	end
end

function ItemScore:ExportMrRobotFormat(setName)
	local ret = ""
	
	-- check parent first
	local effSet = self:GetEffectiveSet(setName)
	
	if not effSet then return ret end
	
	for i, v in pairs(effSet) do
		local statName = statsToMrRobot[i]
		
		if not statName then
			statName = ItemScore.supportedStats[i]
		end
		
		if not statName then statName = i end
		ret = ret .. statName .. " " .. v.score .. "\n"
	end
	
	return ret
end

local pawnToStats = {
	["CritRating"] = "MELEE_CRIT_RATING",
	["MasteryRating"] = "MASTERY_RATING",
	["HasteRating"] = "MELEE_HASTE_RATING",
	["Dps"] = "DPS",
	["SpellPower"] = "SPELL_POWER",
	["Ap"] = "AP",
	["MovementSpeed"] = "RUN_SPEED",
	["BonusArmor"] = "ARMOR_BONUS",
	["ResilienceRating"] = "RESILIENCE_RATING",
	["MaxDamage"] = "MAX_DAMAGE",
	["SpellPenetration"] = "SPELLPEN",
}

local statsToPawn = {}
	
-- now set the values of each index to readable values from stat logic
for i, v in pairs(pawnToStats) do
	statsToPawn[v] = i
end

function ItemScore:ParsePawnFormat(tag)
	local values = {}
	
	-- Get the name
	local name = string.match(tag, "%( *Pawn: *v1: *\"([^\"]+)\" *:")
	
	if not name then 
		DEFAULT_CHAT_FRAME:AddMessage("Malformed Pawn tag. Could not determine name.")
		return
	end
	
	for k, v in string.gmatch(tag, "(%w+) *= *(%d%.?%d*)") do
		values[k] = v
	end
	
	self:CreateNewSet(name)
	for k, v in pairs(values) do
		-- try some hard-coded values first
		local statName = pawnToStats[k]
		
		if not statName then
			statName = ItemScore.reverseStats[k]
		end
		
		if not statName then
			DEFAULT_CHAT_FRAME:AddMessage("Unsupported stat '" .. k .. "' ignored.")
		elseif self:AddStat(name, statName) then
			self:SetScore(name, statName, v)
		end
	end
end

function ItemScore:ExportPawnFormat(setName)
	local ret = ""
	
	-- check parent first
	local effSet = self:GetEffectiveSet(setName)
	
	if not effSet then return ret end
	
	ret = "( Pawn: v1: \"" .. setName .. "\": "
	local first = true
	
	for i, v in pairs(effSet) do
		local statName = statsToPawn[i]
		
		if not statName then
			statName = ItemScore.supportedStats[i]
		end
		
		if not statName then statName = i end
		
		if not first then
			ret = ret .. ", "
		end
		
		first = false
		ret = ret .. statName .. "=" .. v.score
	end
	
	ret = ret .. " )"
	
	return ret
end