
function ItemScore:SetShowTrinketUptime(value)
	self.db.char.showTrinketUptime = value
end

function ItemScore:GetShowTrinketUptime()
	return self.db.char.showTrinketUptime
end

function ItemScore:SetShowTrinketEffect(value)
	self.db.char.showTrinketEffect = value
end

function ItemScore:GetShowTrinketEffect(value)
	return self.db.char.showTrinketEffect
end

function ItemScore:GetTrinketClauses()
	return self.db.global.trinketClauses
end

function ItemScore:GetTrinketClauseEntry(clause)
	local clauses = self:GetTrinketClauses()
	return clauses[clause]
end

function ItemScore:AddTrinketClause(value)
	self.db.global.trinketClauses[value] = { ["All"] = true }
end

function ItemScore:RemoveTrinketClause(value)
	self.db.global.trinketClauses[value] = nil
end

function ItemScore:GetTrinketClauseCategory(clause, category)
	local clauseEntry = self:GetTrinketClauseEntry(clause)
	if clauseEntry then
		return clauseEntry[category]
	end
	
	return nil
end

function ItemScore:SetTrinketClauseCategory(clause, category, value)
	local clauseEntry = self:GetTrinketClauseEntry(clause)
	if clauseEntry then
		clauseEntry[category] = value
	end
end

function ItemScore:MigrateTrinketClauses()
	local clauses = self:GetTrinketClauses()
	for i in pairs(clauses) do
		if type(clauses[i]) == "boolean" then
			self:AddTrinketClause(i)
		end
	end
end

function ItemScore:MakeTrinketMatchClause(clause)
	-- need to escape the parentheses
	local matchClause = string.gsub(clause, "%(", "%%(")
	matchClause = string.gsub(matchClause, "%)", "%%)")
	matchClause = string.gsub(matchClause, "%.", "%%.")
	
	-- ignored words are not captured
	matchClause = string.gsub(matchClause, "%$ignore", "[%%w']+ ?[%%w']*")
	
	-- captured parts are assigned to the $name in the table
	matchClause = string.gsub(matchClause, "%$stat", "(%%w+ ?%%w*)")
	matchClause = string.gsub(matchClause, "%$amount", "(%%d+,?%%d*)")
	matchClause = string.gsub(matchClause, "%$duration", "(%%d+)")
	matchClause = string.gsub(matchClause, "%$chance", "(%%d+)%%%%")
	matchClause = string.gsub(matchClause, "%$maxstacks", "(%%d+)")
	matchClause = string.gsub(matchClause, "%$ppm", "(%%d+%%.?%%d*)")
	matchClause = string.gsub(matchClause, "%$stackfreq", "(%%d+%%.?%%d*)")
	matchClause = string.gsub(matchClause, "%$cooldown", "(%%d+%%.?%%d*)")
	matchClause = string.gsub(matchClause, "%$cdmin", "(%%d+)")
	matchClause = string.gsub(matchClause, "%$cdsec", "(%%d+)")
	return matchClause
end

local function pack(...)
  return {...}
end

function ItemScore:GetTrinketVals(clause, valList)
	local dict = {}
	local vars = string.gmatch(clause, "%$(%w+)")
	ItemScore.test = vars
	if not vars then return dict end
	if not valList then return dict end
	
	local j
	while true do
		local var, val
		var = vars()
		-- ignore the ignore entries
		while var and var == "ignore" do
			var = vars()
		end
		
		j, val = next(valList, j)
		
		local varStr = var or "nil"
		local valStr = val or "nil"
		
		if not var or not val then return dict end
		
		if var and val then
			dict[var] = val
		end
	end
	
	return dict
end

function ItemScore:CalculateTrinketUptime(vals, foundClause)
	--for var, val in pairs(vals) do DEFAULT_CHAT_FRAME:AddMessage(var .. " = " .. val) end
	-- get rid of , character in amount
	
	-- REVISIT - assumes standard 1.5 gcd
	local attackrate = 1.5
	
	local uptime = 0
	if foundClause == "equip" then
		if vals["chance"] and vals["cooldown"] and vals["duration"] then
			local procperiod = vals["cooldown"] + (attackrate / (vals["chance"] / 100))
			uptime = vals["duration"] / procperiod
		elseif vals["ppm"] and vals["maxstacks"] and vals["duration"] then
			-- proc trinket with stacking buff
			uptime = vals["ppm"] * (vals["duration"] / 60)
		elseif vals["ppm"] and vals["duration"] then
			-- proc trinket
			uptime = vals["ppm"] * (vals["duration"] / 60)
		elseif vals["duration"] then
			-- on chance trinket
			-- assumes 20% proc chance with 45s icd
			local chance = 0.2
			local cooldown = 45
			local procperiod = cooldown + (attackrate / 0.2)
			uptime = vals["duration"] / procperiod 
		end
	elseif foundClause == "use" then
		if vals["cooldown"] then
			-- on use trinket
			uptime = vals["duration"] / (vals["cooldown"] * 60)
		elseif vals["cdmin"] and vals["cdsec"] then
			-- on use trinket with min:sec
			uptime = vals["duration"] / ((vals["cdmin"] * 60) + vals["cdsec"])
		end
	end
	
	return uptime
end

function ItemScore:CalculateTrinketStat(statAmount, uptime, maxstacks)
	local maxstackFactor = 1
	if maxstacks then
		maxstackFactor = maxstacks / 2
	end
	return statAmount * uptime * maxstackFactor
end

function ItemScore:CacheTrinketInfo(item, adjustments)
	if not self.trinketCache then self.trinketCache = {} end
	if not self.trinketCache[item] then 
		self.trinketCache[item] = adjustments 
	end
end

function ItemScore:GetCachedTrinketEntry(item)
	if not self.trinketCache then return nil end
	return self.trinketCache[item]
end

function ItemScore:SetAdjustment(adjustmentTable, statToAdjust, adjAmount, categories)
	local entry = {}
	entry.categories = categories
	entry.statToAdjust = statToAdjust
	entry.adjAmount = adjAmount
	table.insert(adjustmentTable, entry)
end

-- returns a table of adjustments and categories for which it applies
function ItemScore:HandleTrinket(item, itemTable)
	local adjustments = {} -- adjustments to attributes
	
	if not self:TrinketCalcEnabled() then
		return adjustments
	end
	
	local tip
	local modTip
	if (type(item) == "string") or (type(item) == "number") then -- common case first
	elseif type(item) == "table" and type(item.GetItem) == "function" then
		tip = item -- must be tooltip - we'll grab this so that we can modify it
		modTip = self:GetShowTrinketUptime() or self:GetShowTrinketEffect()
		-- Get the link
		_, item = item:GetItem()
		if type(item) ~= "string" then return adjustments end
	else
		return adjustments
	end
	
	-- try the cache first
	local cacheEntry = self:GetCachedTrinketEntry(item)
	if cacheEntry and not modTip then
		return cacheEntry
	end
	
	if not tip then
		local _, link = GetItemInfo(item)
		if not link then return adjustments end -- don't know what's going on here
		
		tip = StatLogic.tip
		tip:ClearLines() -- this is required or SetX won't work the second time its called
		tip:SetHyperlink(link)
	end
	
	local trinketClauses = self:GetTrinketClauses()
	
	for i = 2, tip:NumLines() do
		local tipTextLeft = tip:GetName().."TextLeft"
		local fontString = _G[tipTextLeft..i]
		local text = fontString:GetText();
		local foundClause
		if text and string.find(text, "^Equip: ") then foundClause = "equip" end
		if text and string.find(text, "^Use: ") then foundClause = "use" end
		
		if foundClause then
			local foundMatch = false
			for clause in pairs(trinketClauses) do
				-- convert into matchable string
				local matchClause = self:MakeTrinketMatchClause(clause)
				
				if string.find(text, matchClause) then
					foundMatch = true
					local trinketVals = self:GetTrinketVals(clause, pack(string.match(text, matchClause)))
					
					if not trinketVals["amount"] or not trinketVals["stat"] then
						-- can't continue
						break
					end
					
					-- get rid of , character in amount
					local amount = string.gsub(trinketVals["amount"], ",", "")
					amount = tonumber(amount)
					local uptime = self:CalculateTrinketUptime(trinketVals, foundClause)
					local adjAmount = self:CalculateTrinketStat(amount, uptime, trinketVals["maxstacks"])
										
					local statColor = ""
					
					local statToAdj = self.reverseStats[trinketVals["stat"]] or 
					 				  self.trinketStats[string.lower(trinketVals["stat"])] -- trinket stats uses lowercase
					if not statToAdj then
						statColor = "|cFFFF0000"
					end
					
					if modTip then
						local modColor = "|cFF888888"
						local modStr = ""
						if self:GetShowTrinketUptime() then
							modStr = string.format("%d%%", uptime * 100) -- uptime as a percentage
						end
						if self:GetShowTrinketEffect() then
							if modStr ~= "" then
								modStr = modStr .. ", "
							end
							modStr = modStr .. "+" .. string.format("%d", adjAmount) .. " " .. statColor .. trinketVals["stat"] .. modColor
						end
						modStr = modColor .. " (" .. modStr .. ")"
						text = text .. modStr 
						fontString:SetText(text)
					end
					if statToAdj and adjAmount ~= 0 then
						-- don't modify the table, add it to a separate
						-- adjustments table
						--local currentVal = itemTable[statToAdj] or 0 
						--itemTable[statToAdj] = currentVal + adjAmount
						self:SetAdjustment(adjustments, statToAdj, adjAmount, 
										   self:GetTrinketClauseEntry(clause))
					end
					break
				end
				self:CacheTrinketInfo(item, adjustments)
			end
			
			-- if we didn't find a match, let's add it to our possible patterns
			if not foundMatch then
				if not self.cachedClauses then self.cachedClauses = {} end
				self.cachedClauses[text] = true
				if self.ItemScoreConfig then
					self.ItemScoreConfig:UpdateClauseTable() -- update the config list
				end
			end 
		end
	end --for each line in the tooltip
	
	return adjustments
end

function ItemScore:AddTrinketClauses(clauseTable)
	for i in pairs(clauseTable) do
		self.db.global.trinketClauses[i] = clauseTable[i]
	end
end

function ItemScore:AddDefaultTrinketClauses()
	local trinketClauses = {
		["Use: Increases your $stat by $amount for $duration sec. ($cdmin Min $cdsec Sec Cooldown)"] = {
			["All"] = true,
		},
		["Equip: Your spells have a chance to trigger $ignore for $duration sec.  While $ignore is active, you gain $amount $stat every $stackfreq sec, stacking up to $maxstacks times.  (Approximately $ppm procs per minute)"] = {
			["All"] = true,
		},
		["Equip: Your spells have a chance to grant $amount $stat for $duration sec."] = {
			["All"] = true,
		},
		["Equip: Your attacks and beneficial spells have a chance to grant $amount $stat for $duration sec.  (Approximately $ppm procs per minute)"] = {
			["All"] = true,
		},
		["Use: Grants $amount $stat for $duration sec. ($cooldown Min Cooldown)"] = {
			["All"] = true,
		},
		["Equip: Your attacks have a chance to grant $amount $stat for $duration sec.  (Approximately $ppm procs per minute)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Equip: Your healing and damaging spells have a chance to grant $amount $stat for $duration sec."] = {
			["All"] = true,
		},
		["Use: Increases your $stat by $amount for $duration sec. ($cooldown Min Cooldown)"] = {
			["All"] = true,
		},
		["Equip: When your attacks hit you have a chance to gain $amount $stat for $duration sec."] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Equip: Your attacks have a chance to grant $amount $stat for $duration sec.  ($chance chance, $cooldown sec cooldown)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Equip: Your melee attacks have a chance to trigger $ignore for $duration sec.  While $ignore is active, you gain $amount $stat every $stackfreq sec, stacking up to $maxstacks times.  (Approximately $ppm procs per minute)"] = {
			["Melee"] = true,
			["All"] = false,
		},
		["Equip: Each time your attacks hit, you have a chance to gain $amount $stat for $duration sec.  ($chance chance, $cooldown sec cooldown)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Equip: Your attacks have a chance to trigger $ignore for $duration sec.  While $ignore is active, you gain $amount $stat every $stackfreq sec, stacking up to $maxstacks times.  (Approximately $ppm procs per minute)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Use: Grants $amount $stat for $duration sec. ($cdmin Min $cdsec Sec Cooldown)"] = {
			["All"] = true,
		},
		["Equip: When your spells deal critical damage, you have a chance to gain $amount $stat for $duration sec.  (Approximately $ppm procs per minute)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
		["Equip: Your attacks have a chance to grant you $amount $stat for $duration sec.  ($chance chance, $cooldown sec cooldown)"] = {
			["Ranged"] = true,
			["All"] = false,
			["Melee"] = true,
		},
	}
	self:AddTrinketClauses(trinketClauses)
end

function ItemScore:ResetTrinketClauses()
	self.db.global.trinketClauses = {}
	self:AddDefaultTrinketClauses()
end
